@@ -117,6 +117,10 @@ string HTMLRenderer::dump_embedded_font (GfxFont * font, FontInfo & info)
                 {
                     suffix = ".cid";
                 }
+                else if (subtype == "OpenType")
+                {
+                    suffix = ".otf";
+                }
                 else
                 {
                     cerr << "Unknown subtype: " << subtype << endl;
@@ -192,11 +196,7 @@ string HTMLRenderer::dump_type3_font (GfxFont * font, FontInfo & info)
     FT_Library ft_lib;
     FT_Init_FreeType(&ft_lib);
     CairoFontEngine font_engine(ft_lib); 
-#if POPPLER_OLDER_THAN_0_23_0
-    auto * cur_font = font_engine.getFont(font, cur_doc, true);
-#else
     auto * cur_font = font_engine.getFont(font, cur_doc, true, xref);
-#endif
     auto used_map = preprocessor.get_code_map(hash_ref(font->getID()));
 
     //calculate transformed metrics
@@ -398,8 +398,8 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
     GfxCIDFont * font_cid = nullptr;
 
     string suffix = get_suffix(filepath);
-    for(auto iter = suffix.begin(); iter != suffix.end(); ++iter)
-        *iter = tolower(*iter);
+    for(auto & c : suffix)
+        c = tolower(c);
 
     /*
      * if parm->tounicode is 0, try the provided tounicode map first
@@ -487,7 +487,7 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
             unordered_set<string> nameset;
             bool name_conflict_warned = false;
 
-            memset(cur_mapping2, 0, 0x100 * sizeof(char*));
+            std::fill(cur_mapping2.begin(), cur_mapping2.end(), (char*)nullptr);
 
             for(int i = 0; i < 256; ++i)
             {
@@ -516,7 +516,7 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
                 }
             }
 
-            ffw_reencode_raw2(cur_mapping2, 256, 0);
+            ffw_reencode_raw2(cur_mapping2.data(), 256, 0);
         }
     }
     else
@@ -580,8 +580,8 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
         bool name_conflict_warned = false;
 
         auto ctu = font->getToUnicode();
-        memset(cur_mapping, -1, 0x10000 * sizeof(*cur_mapping));
-        memset(width_list, -1, 0x10000 * sizeof(*width_list));
+        std::fill(cur_mapping.begin(), cur_mapping.end(), -1);
+        std::fill(width_list.begin(), width_list.end(), -1);
 
         if(code2GID)
             maxcode = min<int>(maxcode, code2GID_len - 1);
@@ -643,9 +643,8 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
                         retried = true;
                         codeset.clear();
                         info.use_tounicode = false;
-                        //TODO: constant for the length
-                        memset(cur_mapping, -1, 0x10000 * sizeof(*cur_mapping));
-                        memset(width_list, -1, 0x10000 * sizeof(*width_list));
+                        std::fill(cur_mapping.begin(), cur_mapping.end(), -1);
+                        std::fill(width_list.begin(), width_list.end(), -1);
                         cur_code = -1;
                         if(param.debug)
                         {
@@ -704,9 +703,9 @@ void HTMLRenderer::embed_font(const string & filepath, GfxFont * font, FontInfo
             }
         }
 
-        ffw_set_widths(width_list, max_key + 1, param.stretch_narrow_glyph, param.squeeze_wide_glyph);
+        ffw_set_widths(width_list.data(), max_key + 1, param.stretch_narrow_glyph, param.squeeze_wide_glyph);
         
-        ffw_reencode_raw(cur_mapping, max_key + 1, 1);
+        ffw_reencode_raw(cur_mapping.data(), max_key + 1, 1);
 
         // In some space offsets in HTML, we insert a ' ' there in order to improve text copy&paste
         // We need to make sure that ' ' is in the font, otherwise it would be very ugly if you select the text
@@ -887,7 +886,7 @@ const FontInfo * HTMLRenderer::install_font(GfxFont * font)
      * which does not make much sense in our case
      * If we specify gFalse here, font_loc->locaType cannot be gfxFontLocResident
      */
-    if(auto * font_loc = font->locateFont(xref, gFalse))
+    if(auto * font_loc = font->locateFont(xref, nullptr))
     {
         switch(font_loc -> locType)
         {
@@ -942,7 +941,7 @@ void HTMLRenderer::install_external_font(GfxFont * font, FontInfo & info)
         cerr << "Warning: workaround for font names in bad encodings." << endl;
     }
 
-    GfxFontLoc * localfontloc = font->locateFont(xref, gFalse);
+    GfxFontLoc * localfontloc = font->locateFont(xref, nullptr);
 
     if(param.embed_external_font)
     {
@@ -1065,8 +1064,8 @@ void HTMLRenderer::export_local_font(const FontInfo & info, GfxFont * font, cons
     f_css.fs << "font-family:" << ((cssfont == "") ? (original_font_name + "," + general_font_family(font)) : cssfont) << ";";
 
     string fn = original_font_name;
-    for(auto iter = fn.begin(); iter != fn.end(); ++iter)
-        *iter = tolower(*iter);
+    for(auto & c : fn)
+        c = tolower(c);
 
     if(font->isBold() || (fn.find("bold") != string::npos))
         f_css.fs << "font-weight:bold;";
